@@ -7,6 +7,7 @@ import asyncio
 import threading
 import tornado.web
 import tornado.ioloop
+import tornado.websocket
 import tornado.httpserver
 from tornado.httputil import HTTPHeaders
 
@@ -474,6 +475,61 @@ def special_service(request, output):
     output['finish'] = True
 
 
+class WebSocketSever(tornado.websocket.WebSocketHandler):
+    opened_connections = {}
+
+    def open(self):
+        pass
+
+    def on_message(self, message):
+        try:
+            msg = json.loads(message, encoding='utf-8')
+        except Exception as e:
+            logging.error(str(e))
+            logging.error('WebSocket消息必须采用JSON格式！')
+            return
+
+        if msg.get('type') == 'set_conn_id':
+            self.conn_id = msg.get('content')
+            if self.conn_id is not None:
+                old = self.opened_connections.get(self.conn_id)
+                if old is None:
+                    self.opened_connections[self.conn_id] = self
+                    logging.critical('WebSocket connection opened: {}'.format(self.conn_id))
+                    self.write_message({
+                        'type': 'welcome',
+                        'content': 'WebSocket connection opened: {}'.format(self.conn_id),
+                    })
+            else:
+                logging.error('WebSocket conn_id error')
+                self.close()
+        else:
+            receiver = msg.get('receiver')
+            # 未指定receiver或receiver第一个字符为'/'
+            if receiver is None or receiver[0] == '/':
+                return
+            if '/' in receiver:
+                i = receiver.find('/')
+                msg['receiver'] = receiver[i + 1:]
+                receiver = receiver[0: i]
+
+            # 转发消息
+            next_websocket = self.opened_connections.get(receiver)
+            if next_websocket:
+                msg['sender'] = self.conn_id
+                next_websocket.write_message(msg)
+
+    def on_close(self):
+        self.opened_connections.pop(self.conn_id)
+        logging.critical('WebSocket connection closed: {}'.format(self.conn_id))
+
+    def check_origin(self, origin):
+        return True
+
+    def allow_draft76(self):
+        return True
+
+
 class HealthChecker(tornado.web.RequestHandler):
     def get(self, *args, **kwargs):
         self.write('{"description": "Micro-service Discovery Client", "status": "UP"}')
@@ -544,6 +600,7 @@ def start():
         handlers = [
             (r'/management/health', HealthChecker),
             (r'/api/data', DataApi),
+            (r'/websocket', WebSocketSever),
             (r'/(.*)', GatewayApi),
         ]
     else:
